@@ -125,22 +125,6 @@ class GeoCoord:
     def __ceil__(self):
         return GeoCoord(math.ceil(self.decimal_degree))
         
-    def __add__(self, other):
-        # other is a GeoCoord object
-        return GeoCoord(self.decimal_degree + other.decimal_degree)
-    
-    def __iadd__(self, other):
-        # other is a GeoCoord object
-        return self.__add__(other)
-    
-    def __sub__(self, other):
-        # other is a GeoCoord object
-        return self.__add__(-other)
-    
-    def __isub__(self, other):
-        # other is a GeoCoord object
-        return self.__add__(-other)
-        
     def __int__(self):
         return self.degree
     
@@ -150,21 +134,14 @@ class GeoCoord:
     def __str__(self):
         return str(self.decimal_degree)
     
-    def __mul__(self, other):
-        # other is a scalar
-        return GeoCoord(self.decimal_degree*other)
+    def __repr__(self):
+        return self.__str__()
     
-    def __imul__(self, other):
-        # other is a scalar
-        return self.__mul__(other)
-    
-    def __div__(self, other):
-        # other is a scalar
-        return GeoCoord(self.decimal_degree/other)
-    
-    def __idiv__(self, other):
-        # other is a scalar
-        return self.__div__(other)
+    def type(self):
+        '''
+        Identifies the object type
+        '''
+        return 'GeoCoord'
 
 class Latitude(GeoCoord):
     '''
@@ -193,6 +170,9 @@ class Latitude(GeoCoord):
             self._update()
         else:
             raise ValueError('Hemisphere identifier for latitudes must be N or S')
+        
+    def __repr__(self):
+        return 'Latitude %s' %(self.__str__())
 
 class Longitude(GeoCoord):
     '''
@@ -221,6 +201,9 @@ class Longitude(GeoCoord):
             self._update()
         else:
             raise(ValueError, 'Hemisphere identifier for longitudes must be E or W')
+    
+    def __repr__(self):
+        return 'Longitude %s' %(self.__str__())
 
 def string2geocoord(coord_str, coord_class, format_str = 'D'):
     '''
@@ -301,70 +284,43 @@ class LatLon:
         Return lat/lon pairs as complex coordinates
         '''
         return self.lat.decimal_degree + 1j * self.lon.decimal_degree
-    
-    def __add__(self, other):
-        # other is a LatLon object
-        return LatLon(self.lat + other.lat, self.lon + other.lon)
-    
-    def __iadd__(self, other):
-        # other is a LatLon object
-        return self.__add__(other)
-    
-    def __sub__(self, other):
-        # other is a LatLon object
-        return LatLon(self.lat - other.lat, self.lon - other.lon)
-    
-    def __isub__(self, other):
-        # other is a LatLon object
-        return self.__sub__(other)
-    
-    def __mul__(self, other):
-        # other is a scalar
-        return LatLon(self.lat*other, self.lon*other)
-    
-    def __imul__(self, other):
-        return self.__mul__(other)
-    
-    def __div__(self, other):
-        # other is a scalar
-        return LatLon(self.lat/other, self.lon/other)
-    
-    def __idiv__(self, other):
-        # other is a scalar
-        return self.__div__(other)
-    
-    def heading_initial(self, other, ellipse = 'WGS84'):
+
+    def _pyproj_inv(self, other, ellipse = 'WGS84'):
+        '''
+        Perform Pyproj's inv operation on two LatLon objects
+        Returns the initial heading and reverse heading in degrees, and the distance
+        in km.
+        '''
+        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
+        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
+        g = pyproj.Geod(ellps = ellipse)
+        heading_initial, heading_reverse, distance = g.inv(lon1, lat1, lon2, lat2, radians = False)
+        distance = distance/1000.0
+        return {'heading_initial': heading_initial, 'heading_reverse': heading_reverse, 'distance': distance}
+        
+    def heading_initial(self, other, **kwargs):
         '''
         Returns initial bearing between two LatLon objects in degrees using pyproj.
         Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
         for the FAI ellipsoid.
         '''
-        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
-        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
-        g = pyproj.Geod(ellps = ellipse)
-        return g.inv(lon1, lat1, lon2, lat2, radians = False)[0]
+        return self._pyproj_inv(other, **kwargs)['heading_initial']
     
-    def heading_reverse(self, other, ellipse = 'WGS84'):
+    def heading_reverse(self, other, **kwargs):
         '''
         Returns reverse bearing between two LatLon objects in degrees using pyproj.
         Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
         for the FAI ellipsoid.
         '''
-        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
-        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
-        g = pyproj.Geod(ellps = ellipse)
-        return g.inv(lon1, lat1, lon2, lat2, radians = False)[1]
+        return self._pyproj_inv(other, **kwargs)['heading_reverse']
         
-    def distance(self, other, ellipse = 'WGS84'):
+    def distance(self, other, **kwargs):
         '''
         Returns great circle distance between two LatLon objects in km using pyproj.
         Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
         for the FAI ellipsoid.
         '''
-        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
-        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
-        g = pyproj.Geod(ellps = ellipse)
-        return g.inv(lon1, lat1, lon2, lat2, radians = False)[2]/1000.0
+        return self._pyproj_inv(other, **kwargs)['distance']
         
     def distance_sphere(self, other, radius = 6371.0):
         '''
@@ -407,6 +363,57 @@ class LatLon:
         using the format specified by formatter
         '''
         return (self.lat.to_string(formatter), self.lon.to_string(formatter))
+        
+    def _sub_vector(self, other):
+        '''
+        Called when subtracting a GeoVector object from self
+        '''
+        heading, distance = other()
+        heading = (heading + 180)%360 # Flip heading
+        new_latlon = LatLon(self.lat, self.lon) # Copy current position
+        return new_latlon.offset(heading, distance) # Offset position by GeoVector
+        
+    def _sub_latlon(self, other):
+        '''
+        Called when subtracting a LatLon object from self
+        '''
+        inv = self._pyproj_inv(other)
+        heading = inv['heading_reverse']
+        distance = inv['distance']
+        return GeoVector(initial_heading = heading, distance = distance)
+        
+    def __add__(self, other):
+        # other is a GeoVector object
+        heading, distance = other()
+        new_latlon = LatLon(self.lat, self.lon)
+        return new_latlon.offset(heading, distance)
+    
+    def __iadd__(self, other):
+        # other is a GeoVector object
+        return self.__add__(other)
+
+    def __sub__(self, other):
+        # if other is a GeoVector, will return LatLon object
+        # if other is a LatLon, will return GeoVector object
+        object_operator = {'GeoVector': self._sub_vector,
+                           'LatLon': self._sub_latlon}
+        return object_operator[other.type()](other)
+    
+    def __isub__(self, other):
+        # other is a GeoVector object
+        return self.__sub__(other)
+    
+    def __str__(self):
+        return '%s, %s' %(self.lat.__str__(), self.lon.__str__())
+    
+    def __repr__(self):
+        return '%s, %s' %(self.lat.__repr__(), self.lon.__repr__())
+    
+    def type(self):
+        '''
+        Identifies the object type
+        '''
+        return 'LatLon'
 
 def string2latlon(lat_str, lon_str, format_str):
     '''
@@ -425,6 +432,102 @@ def string2latlon(lat_str, lon_str, format_str):
     new_latlon = LatLon(lat = lat, lon = lon)
     return new_latlon
 
+class GeoVector:
+    '''
+    Object representing the distance and heading between two lat/lon coordinates
+    Can be created by:
+        1. Passing dx and dy arguments
+        2. Passing initial_heading and distance keyword arguments
+        3. Subtracting two LatLon objects
+    '''
+    def __init__(self, dx = None, dy = None, initial_heading = None, distance = None):
+        '''
+        Create a GeoVector object
+        Inputs:
+            dx (scalar) - the zonal component of a vector in km
+            dy (scalar) - the meridional component of a vector in km
+            initial_heading (scalar) - the initial heading of the vector in degrees
+            distance (scalar) - the magnitude of the vector in km
+        '''
+        if dx == None and dy == None: # If only initial_heading and distance are given
+            self.theta = initial_heading
+            theta_rad = math.radians(self.theta)
+            self.magnitude = distance
+            self.dx = self.magnitude * math.cos(theta_rad)
+            self.dy = self.magnitude * math.sin(theta_rad)
+        elif initial_heading == None and distance == None: # If only dx and dy are given
+            self.dx = dx
+            self.dy = dy
+            self._update()
+        else:
+            raise NameError, 'Class GeoVector requires two arguments (dx and dy or initial_heading and distance)'
+        
+    def __call__(self):
+        return self.theta, self.magnitude
+    
+    def _update(self):
+        '''
+        Calculate heading and distance from dx and dy
+        '''
+        self.theta = math.degrees(math.atan(float(self.dy)/self.dx))
+        self.magnitude = math.cos(self.theta) * self.dx
+        
+    def __neg__(self):
+        return GeoVector(-self.dx, -self.dy)
+    
+    def __add__(self, other):
+        # other is a GeoVector object
+        return GeoVector(self.dx + other.dx, self.dy + other.dy)
+    
+    def __iadd__(self, other):
+        # other is a GeoVector object
+        return self.__add__(other)
+    
+    def __sub__(self, other):
+        # other is a GeoVector object
+        return self.__add__(-other)
+    
+    def __isub__(self, other):
+        # other is a GeoVector object
+        return self.__sub__(other)
+    
+    def __cmp__(self, other):
+        return cmp(self.magnitude, other.magnitude)
+    
+    def __pos__(self):
+        return GeoVector(self.dx, self.dy)
+        
+    def __abs__(self):
+        return self.__pos__()
+
+    def __mul__(self, other):
+        # other is a scalar
+        return GeoVector(initial_heading = self.theta, distance = self.magnitude * other)
+    
+    def __imul__(self, other):
+        # other is a scalar
+        return self.__mul__(other)
+    
+    def __div__(self, other):
+        # other is a scalar
+        return GeoVector(initial_heading = self.theta, distance = self.magnitude / other)
+    
+    def __idiv__(self, other):
+        # other is a scalar
+        return self.__div__(other)
+    
+    def __str__(self):
+        return '%s %s' %(self.theta, self.magnitude)
+    
+    def __repr__(self):
+        return 'Heading %s, Distance %s' %(self.theta, self.magnitude)
+    
+    def type(self):
+        '''
+        Identifies the object type
+        '''
+        return 'GeoVector'
+
 def test():
     palmyra = LatLon(Latitude(5.8833), Longitude(-162.0833)) # Simplest case - initialize from decimal degrees
     print palmyra.to_string('d% %m% %S% %H') # Print coordinates to degree minute second (returns ('5 52 59.88 N', '162 4 59.88 W'))
@@ -440,6 +543,10 @@ def test():
     print initial_heading
     hnl = palmyra.offset(initial_heading, distance) # Reconstruct lat/lon for Honolulu based on offset from Palmyra
     print hnl.to_string('D') # Coordinates of Honolulu are latitude 21.3, longitude -157.8167
+    vector = (honolulu - palmyra) * 2 # A GeoVector with heading equal to the vector between palmyra and honolulu, but 2x the magnitude
+    print vector # Print heading and magnitude
+    print palmyra + (vector/2.0) # Recreate the coordinates of Honolulu by adding half of vector to palmyra
+    print 'Finished test!'
 
 if __name__ == '__main__':
     test()
